#ifndef GAME_H
#define GAME_H

#include "glstuff.h"

int cube_text, field_text;

const int Cube_Size = 32;
const float Rotation_Speed = 0.166;
//const float Move_Speed = 0.237;
const float Move_Speed = 0.1;

typedef struct CUBE_T Cube, *Cube_P;

/*typedef*/ struct CUBE_T {
    float x;
    float y;    
    float angle;
    int ID;
    Cube_P target;
    float delta_angle;
    float rotated;
    float max_rotation;
    int is_moving;
    int can_move;
    float dx;
    float dy;
    int enabled;
    float ax;
    float ay;
} /*Cube, *Cube_P*/;

Cube cubes[64];

float rads_to_degs(float rads)
{
    return rads * 180 / M_PI;
}

float degs_to_rads(float degs)
{
    return degs * M_PI / 180;
}

int create_cubes()
{
    int y; 
    int x;
    Cube_P cube;
    for(y = 0; y < 8; ++y) {
        for(x = 0; x < 8; ++x) {
            cube = &cubes[y*8+x];
            cube->y = -(32+y*32*2)+W/2;
            cube->x = 32+x*32*2-W/2;
            cube->angle = rand() % 360; 
            if (cube->angle > 180) {
                cube->angle = - (360 - cube->angle);
            }
            cube->ID = y*8 + x; 
            cube->target = 0;
            cube->enabled = 1;
            cube->can_move = 1;
            cube->is_moving = 0;
        
            //cube->enabled = 0;
          
        }
    }
    return 1;
}

/*
int create_cubes()
{
    int y; 
    int x;
    Cube_P cube;
    for(y = 0; y < 4; ++y) {
        for(x = 0; x < 4; ++x) {
            cube = &cubes[y*8+x];
            cube->y = -(64+y*64*2)+W/2;
            cube->x = 64+x*64*2-W/2;
            cube->angle = rand() % 360; 
            if (cube->angle > 180) {
                cube->angle = - (360 - cube->angle);
            }
            cube->ID = y*8 + x; 
            cube->target = 0;
            cube->enabled = 1;
            cube->can_move = 1;
            cube->is_moving = 0;
        
            //cube->enabled = 0;
          
        }
    }
    return 1;
}
*/

int game_restart()
{
    create_cubes();
    return 1;
}

int cube_select_target(Cube_P cube)
{
    int i;
    Cube_P target;
    float x;
    float y;
    float cosB;
    float B;
    float angles[64];
    float res_angle;
    int res_ID;
    float angle;
    for(i = 0; i < 64; ++i) 
    {
        target = &cubes[i];
        angles[i] = 777;
        if(!target->enabled)
            continue;
        if(target->ID == cube->ID)
            continue;
        if(target->is_moving)
            continue;
        x = target->x - cube->x;
        y = target->y - cube->y;
        cosB = y / sqrtf(x*x + y*y);
        B = acos(cosB);
        B = rads_to_degs(B);
        if(x < 0) {
            B = -B;
        }
        angles[i] = B - cube->angle;
    }
    res_angle = 777;
    res_ID = 64;
    for(i = 0; i < 64; ++i) 
    {
        angle = angles[i];
        if(fabs(angle) < fabs(res_angle)) {
            res_angle = angle;
            res_ID = i;
        }
    }
    if(res_ID == 64) {
        game_restart();
        return 1;
    }
    cube->target = &cubes[res_ID];
    cube->delta_angle = Rotation_Speed;
    if(res_angle < 0) {
        cube->delta_angle = - cube->delta_angle;
    }
    cube->max_rotation = fabs(res_angle);
    cube->rotated = 0;
    //printf("Selected target, angle %f, ID %d -> %d\n", 
        //res_angle, cube->ID, res_ID);
    return 1;
}

int cubes_select_target()
{
    int i;
    for(i = 0; i < 64; ++i) 
    {
        if(!cubes[i].enabled)
            continue;
        if(cubes[i].target != 0)
            continue;
        if(!cubes[i].can_move)
            continue;
        cube_select_target(&cubes[i]);
    }
    return 1;
}

float normalize_angle(float angle)
{
    float from;
    if(fabs(angle) <= 180)
        return angle;
    if(angle > 0) {
        from = -360;
    }
    if(angle < 0) {
        from = 360;
    }
    while(fabs(angle) > 180) {
        angle = from + angle;
    }
    return angle;
}

int cubes_rotate()
{
    int i;
    Cube_P cube;
    
    //float x; float y;
    float A_rad; float B_rad;
    
    for(i = 0; i < 64; ++i) 
    {
        cube = &cubes[i];
        if(!cube->enabled)
            continue;
        if(cube->target == 0)
            continue;
        if(cube->target->is_moving) {
            cube->target = 0;
            continue;
        }
        if(!cube->can_move) {
            continue;
        }
        if(cube->is_moving)
            continue;
        cube->angle += cube->delta_angle;
        cube->angle = normalize_angle(cube->angle);
        cube->rotated += fabs(cube->delta_angle);
        if(cube->rotated >= cube->max_rotation) {
            cube->is_moving = 1;
            cube->target->can_move = 0;
            
            A_rad = degs_to_rads(cube->angle);
            B_rad = degs_to_rads(90 - cube->angle);
            cube->dy = Move_Speed * cos(A_rad);
            cube->dx = Move_Speed * cos(B_rad);
            
            cube->ay = cube->dy / 10;
            cube->ax = cube->dx / 10;
            //cube->dx = sqrtf(Move_Speed*Move_Speed-
            //cube->dy*cube->dy);
            /*cube->dx = fabs(cube->dx);
            cube->dy = fabs(cube->dy);
            x = cube->target->x - cube->x;
            y = cube->target->y - cube->y;
            if(x < 0)
                cube->dx = -cube->dx;
            if(y < 0)
                cube->dy = -cube->dy;
            */
            //printf("Cube %d begins moving by (%f, %f)\n", 
                //cube->ID, cube->dx, cube->dy);
            //printf("Coords of %d: (%f, %f)\n",
                //cube->ID, cube->x, cube->y);
            //printf("Coords of %d: (%f, %f)\n", 
                //cube->target->ID, cube->target->x, 
                //cube->target->y);
        }
    }
    return 1;
}

int cube_stop_atackers(int ID)
{
    int i;
    Cube_P cube;
    for(i = 0; i < 64; ++i)
    {
        cube = &cubes[i];
        if(!cube->enabled)
            continue;
        if(!cube->target)
            continue;
        if(cube->target->ID != ID) 
            continue;
        cube->target = 0;
        cube->is_moving = 0;
    }
    return 1;
}

int cube_check_crash(Cube_P cube)
{
    int i;
    Cube_P target;
    float x;
    float y;
    for(i = 0; i < 64; ++i) 
    {
        target = &cubes[i];
        if(!target->enabled)
            continue;
        if(target->ID == cube->ID)
            continue;
        x = fabs(cube->x - target->x);
        y = fabs(cube->y - target->y);    
        if((x > 16) || (y > 16)) 
            continue;
        target->enabled = 0;
        ////////////
        if(target->target) {
            target->target->can_move = 1;
        }
        cube_stop_atackers(target->ID);
        cube->is_moving = 0;
        cube->target = 0;
        //printf("Cube %d crashed by %d\n", 
            //target->ID, cube->ID);
    }
    return 1;
}

int cubes_move()
{
    int i;
    Cube_P cube;
    for(i = 0; i < 64; ++i) 
    {
        cube = &cubes[i];
        if(!cube->enabled)
            continue;
        if(!cube->is_moving)
            continue;
        cube->x += cube->dx;
        cube->y += cube->dy;
        if(fabs(2*cube->ax*10) > fabs(cube->dx)) {
            cube->dx += cube->ax;
            cube->dy += cube->ay;
        }
        cube_check_crash(cube);
    }
    return 1;
}

/*
Из вправо вниз конвертируем в вправо вверх из центра так:
вправо уменьшаем на половину системы
вниз инвертируем и увеличиваем на половину системы
*/

int convert_x_to_center(int x)
{
    return x - W/2;
}

int convert_y_invert_to_center(int y)
{
    return -y + W/2;
}

int game_init()
{	   
    srand(SDL_GetTicks());
    float ambient[] = {1, 1, 1, 1}; 
    load_image(&field_text, "./data/field.tga", NotAlpha);
    load_image(&cube_text, "./data/cube.tga", NotAlpha);  
    create_cubes();  
	glClearColor(0.1, 0.4, 0.3, 0);
	glColor3f(1.0f, 0.0f, 0.0f);   
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);               
	glViewport(0, 0, W, W);
	glMatrixMode(GL_PROJECTION); 
    glLoadIdentity();
    //l r b t 
    glOrtho(-W/2, W/2, -W/2, W/2, -1000, 1000);                            
    glMatrixMode(GL_MODELVIEW);
	return 1;   
}

int draw_square(int x, int y, int w)
{
    int left = x;
    int right = left + w;
    int top = y;
    int bottom = y - w;
    glBegin(GL_QUADS);
	glTexCoord2i(0, 1);
	glVertex2i(left, top);
	glTexCoord2i(1, 1);
	glVertex2i(right, top);
	glTexCoord2i(1, 0);
	glVertex2i(right, bottom);
	glTexCoord2i(0, 0);
	glVertex2i(left, bottom);
	glEnd();
	return 1;
}

int draw_cubes()
{
    int i;
    Cube_P cube;
    glBindTexture(GL_TEXTURE_2D, cube_text);
    for(i = 0; i < 64; ++i) {
        cube = &cubes[i];
        if(!cube->enabled)
            continue;
        glTranslatef(cube->x, cube->y, 0);
        //glTranslatef(100, 100, 0);
        glRotatef(-cube->angle, 0, 0, 1);
        //glRotatef(10, 0, 0, 1);
	    draw_square(-16, 16, 32);
	    //draw_square(-4, 4, 8);
	    glLoadIdentity();
	    //break;
    }
    return 1;
}

int game_main()
{
    int x, y;
    float ambient[] = {0.15, 0.15, 0.15, 1};
    
    cubes_select_target();
    cubes_rotate();
    cubes_move();
    
	glClear(GL_COLOR_BUFFER_BIT);
	
	glLoadIdentity();
	// eye x y z, center x y z, up x y z
	gluLookAt(0, 0, 1,  0, 0, 0,  0, 1, 0);
	
	glEnable(GL_LIGHTING); 
	glMaterialfv(GL_FRONT, GL_AMBIENT, ambient); 
	glDisable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	
	glBindTexture(GL_TEXTURE_2D, field_text);
	for(x = -W/2; x < W/2; x+=64) {
	   for(y = W/2; y > -W/2; y-=64) {
	       draw_square(x, y, 64);
	   }
	}

	draw_cubes();
	
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	SDL_GL_SwapBuffers();
	gl_print_error();
	wait();
	print_fps();
	return 1;
}

#endif
